@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-#
 # texttable - module for creating simple ASCII tables
 # Copyright (C) 2003-2015 Gerome Fournier <jef(at)foutaise.org>
 #
@@ -72,11 +70,13 @@ Result:
     mnop   0.023    5.000e+78   92    1.280e+22
 """
 
+from __future__ import division
+
 __all__ = ["Texttable", "ArraySizeError"]
 
 __author__ = 'Gerome Fournier <jef(at)foutaise.org>'
 __license__ = 'LGPL'
-__version__ = '0.8.7'
+__version__ = '0.8.8'
 __credits__ = """\
 Jeff Kowalczyk:
     - textwrap improved import
@@ -106,6 +106,7 @@ frinkelpi:
 
 import sys
 import string
+import unicodedata
 
 try:
     if sys.version >= '2.3':
@@ -121,36 +122,40 @@ except ImportError:
 if sys.version >= '2.7':
     from functools import reduce
 
+if sys.version >= '3.0':
+    unicode_type = str
+    bytes_type = bytes
+else:
+    unicode_type = unicode
+    bytes_type = str
+
+
 def obj2unicode(obj):
     """Return a unicode representation of a python object
     """
-    try:
-        if sys.version >= '3.0':
-            return str(obj)
-        else:
-            if isinstance(obj, unicode):
-                return obj
-            else:
-                return str(obj).decode('utf')
-    except UnicodeDecodeError as strerror:
-        sys.stderr.write("UnicodeDecodeError exception for string '%s': %s\n" % (obj, strerror))
-        if sys.version >= '3.0':
-            return str(obj, 'utf', 'replace')
-        else:
-            return str(obj).decode('utf', 'replace')
+    if isinstance(obj, unicode_type):
+        return obj
+    elif isinstance(obj, bytes_type):
+        try:
+            return unicode_type(obj, 'utf-8')
+        except UnicodeDecodeError as strerror:
+            sys.stderr.write("UnicodeDecodeError exception for string '%s': %s\n" % (obj, strerror))
+            return unicode_type(obj, 'utf-8', 'replace')
+    else:
+        return unicode_type(obj)
+
 
 def len(iterable):
     """Redefining len here so it will be able to work with non-ASCII characters
     """
-    if not isinstance(iterable, str):
-        return iterable.__len__()
-
-    try:
-        if sys.version >= '3.0':
-            return len(str)
+    if isinstance(iterable, bytes_type) or isinstance(iterable, unicode_type):
+        unicode_data = obj2unicode(iterable)
+        if hasattr(unicodedata, 'east_asian_width'):
+            w = unicodedata.east_asian_width
+            return sum([w(c) in 'WF' and 2 or 1 for c in unicode_data])
         else:
-            return len(unicode(iterable, 'utf'))
-    except:
+            return unicode_data.__len__()
+    else:
         return iterable.__len__()
 
 
@@ -522,11 +527,25 @@ class Texttable:
                     maxi[i] = max(maxi[i], self._len_cell(cell))
                 except (TypeError, IndexError):
                     maxi.append(self._len_cell(cell))
-        items = len(maxi)
-        length = reduce(lambda x, y: x+y, maxi)
-        if self._max_width and length + items * 3 + 1 > self._max_width:
-            maxi = [(self._max_width - items * 3 -1) // items \
-                    for n in range(items)]
+
+        ncols = len(maxi)
+        content_width = sum(maxi)
+        deco_width = 3*(ncols-1) + [0,4][self._has_border()]
+        if self._max_width and (content_width + deco_width) > self._max_width:
+            """ content too wide to fit the expected max_width
+            let's recompute maximum cell width for each cell
+            """
+            if self._max_width < (ncols + deco_width):
+                raise ValueError('max_width too low to render data')
+            available_width = self._max_width - deco_width
+            newmaxi = [0] * ncols
+            i = 0
+            while available_width > 0:
+                if newmaxi[i] < maxi[i]:
+                    newmaxi[i] += 1
+                    available_width -= 1
+                i = (i + 1) % ncols
+            maxi = newmaxi
         self._width = maxi
 
     def _check_align(self):
@@ -558,15 +577,15 @@ class Texttable:
                 if isheader:
                     align = "c"
                 if align == "r":
-                    out += "%s " % (fill * space + cell_line)
+                    out += fill * space + cell_line
                 elif align == "c":
-                    out += "%s " % (int(fill/2) * space + cell_line \
+                    out += (int(fill/2) * space + cell_line \
                             + int(fill/2 + fill%2) * space)
                 else:
-                    out += "%s " % (cell_line + fill * space)
+                    out += cell_line + fill * space
                 if length < len(line):
-                    out += "%s " % [space, self._char_vert][self._has_vlines()]
-            out += "%s\n" % ['', self._char_vert][self._has_border()]
+                    out += " %s " % [space, self._char_vert][self._has_vlines()]
+            out += "%s\n" % ['', space + self._char_vert][self._has_border()]
         return out
 
     def _splitit(self, line, isheader):
